package sample.assembler;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import flex.data.DataDestination;
import flex.data.DataSyncException;
import flex.data.assemblers.AbstractAssembler;
import flex.data.config.MetadataSettings;
import flex.messaging.config.ConfigMap;
import flex.messaging.io.amf.ASObject;

public class SampleAssembler extends AbstractAssembler {
	private String jdbcDriver = "org.hsqldb.jdbcDriver";

	private String jdbcURL;

	private List identityColumnNames;

	private String whereClause = " WHERE PERSON_ID=?";

	public void initialize(String id, ConfigMap properties) {
		MetadataSettings metadataSettings = DataDestination.getDataDestination(
				id).getPropertySettings();
		identityColumnNames = metadataSettings.getIdentityPropertyNames();

		jdbcURL = properties.getPropertyAsMap("database", null)
				.getPropertyAsString("url", "");
		try {
			Class.forName(jdbcDriver);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	public Collection fill(List fillParameters) {
		Connection connection = null;
		try {
			connection = getConnection();
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT * FROM person");
			return resultSetToList(rs);
		} catch (SQLException e) {
			throw new RuntimeException(e);
		} finally {
			close(connection);
		}
	}

	public void updateItem(Object newVersion, Object oldVersion, List changes) {
		ASObject asObject = (ASObject) newVersion;
		Object[] columnNames = asObject.keySet().toArray();
		StringBuffer sql = new StringBuffer("UPDATE person SET ");
		int columnCount = columnNames.length;
		for (int i = 0; i < columnCount; i++) {
			sql.append(columnNames[i] + "=?,");
		}
		sql.deleteCharAt(sql.length() - 1);

		sql.append(whereClause);

		Connection connection = null;
		try {
			connection = getConnection();
			PreparedStatement ps = connection.prepareStatement(sql.toString());
			int i;
			for (i = 0; i < columnCount; i++) {
				ps.setObject(i + 1, asObject.get(columnNames[i]));
			}
			for (int j = 0; j < identityColumnNames.size(); i++, j++) {
				ps.setObject(i + 1, asObject.get(identityColumnNames.get(j)));
			}
			if (ps.executeUpdate() < 1) {
				throw new DataSyncException(getItem((Map) newVersion), null);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			close(connection);
		}
	}

	public void createItem(Object item) {
		ASObject asObject = (ASObject) item;
		Object[] columnNames = asObject.keySet().toArray();
		StringBuffer sql = new StringBuffer("INSERT INTO person");
		sql.append(" (");
		for (int i = 0; i < columnNames.length; i++) {
			sql.append(columnNames[i] + ",");
		}
		sql.deleteCharAt(sql.length() - 1);
		sql.append(") VALUES (");
		for (int i = 0; i < columnNames.length; i++) {
			sql.append("?,");
		}
		sql.deleteCharAt(sql.length() - 1);
		sql.append(")");

		Connection connection = null;
		try {
			connection = getConnection();
			PreparedStatement ps = connection.prepareStatement(sql.toString());
			int i;
			for (i = 0; i < columnNames.length; i++) {
				ps.setObject(i + 1, asObject.get(columnNames[i]));
			}
			ps.executeUpdate();

			String identityColumn = (String) identityColumnNames.get(0);
			sql = new StringBuffer("SELECT max(");
			sql.append("PERSON_ID");
			sql.append(") FROM person");
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(sql.toString());
			rs.next();
			asObject.put(identityColumn, new Integer(rs.getInt(1)));
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			close(connection);
		}
	}

	public void deleteItem(Object item) {
		Map object = (Map) item;
		StringBuffer sql = new StringBuffer("DELETE FROM person");
		sql.append(whereClause);
		
		Connection connection = null;
		try {
			connection = getConnection();
			PreparedStatement ps = connection.prepareStatement(sql.toString());

			int identityColumnCount = identityColumnNames.size();
			for (int i = 0; i < identityColumnCount; i++) {
				ps.setObject(i + 1, object.get(identityColumnNames.get(i)));
			}
			if (ps.executeUpdate() < 1) {
				throw new DataSyncException(getItem((Map) item), null);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			close(connection);
		}
	}

	public Object getItem(Map id) {
		StringBuffer sql = new StringBuffer("SELECT * FROM person");
		sql.append(whereClause);
		Connection connection = null;
		try {
			connection = getConnection();
			PreparedStatement ps = connection.prepareStatement(sql.toString());

			int identityColumnCount = identityColumnNames.size();
			for (int i = 0; i < identityColumnCount; i++) {
				ps.setObject(i + 1, id.get(identityColumnNames.get(i)));
			}
			ResultSet rs = ps.executeQuery();
			List list = resultSetToList(rs);
			
			if (list.size() == 1) {
				return list.get(0);
			} else {
				return null;
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		} finally {
			close(connection);
		}
	}

	private List resultSetToList(ResultSet rs) throws SQLException {
		ResultSetMetaData rsmd = rs.getMetaData();
		int columnCount = rsmd.getColumnCount();
		
		List list = new ArrayList();
		HashMap item = null;
		while (rs.next()) {
			item = new HashMap();
			for (int i = 1; i <= columnCount; i++) {
				item.put(rsmd.getColumnName(i), rs.getString(i));
			}
			list.add(item);
		}
		return list;
	}

	private Connection getConnection() throws SQLException {
		return DriverManager.getConnection(jdbcURL);
	}

	private void close(Connection connection) {
		try {
			connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

}